@@ -35,11 +35,15 @@
 #include "scrollbar.h"
 #include "enums.h"
 #include "utils.h"
+#include "focusable.h"
 
 /* Define this class in GObject system */
-G_DEFINE_TYPE(XfdashboardViewpad,
-				xfdashboard_viewpad,
-				XFDASHBOARD_TYPE_ACTOR)
+static void _xfdashboard_viewpad_focusable_iface_init(XfdashboardFocusableInterface *iface);
+
+G_DEFINE_TYPE_WITH_CODE(XfdashboardViewpad,
+						xfdashboard_viewpad,
+						XFDASHBOARD_TYPE_BACKGROUND,
+						G_IMPLEMENT_INTERFACE(XFDASHBOARD_TYPE_FOCUSABLE, _xfdashboard_viewpad_focusable_iface_init))
 
 /* Private structure - access only by public API if needed */
 #define XFDASHBOARD_VIEWPAD_GET_PRIVATE(obj) \
@@ -407,20 +411,18 @@ static void _xfdashboard_viewpad_on_allocation_changed(ClutterActor *inActor,
 }
 
 /* Scroll to requested position in view */
-static void _xfdashboard_viewpad_on_view_scroll_to(ClutterActor *inActor,
+static void _xfdashboard_viewpad_on_view_scroll_to(XfdashboardViewpad *self,
 													gfloat inX,
 													gfloat inY,
 													gpointer inUserData)
 {
-	XfdashboardViewpad			*self;
 	XfdashboardViewpadPrivate	*priv;
 	XfdashboardView				*view;
 	gfloat						x, y, w, h;
 
-	g_return_if_fail(XFDASHBOARD_IS_VIEWPAD(inActor));
+	g_return_if_fail(XFDASHBOARD_IS_VIEWPAD(self));
 	g_return_if_fail(XFDASHBOARD_IS_VIEW(inUserData));
 
-	self=XFDASHBOARD_VIEWPAD(inActor);
 	priv=self->priv;
 	view=XFDASHBOARD_VIEW(inUserData);
 
@@ -450,6 +452,91 @@ static void _xfdashboard_viewpad_on_view_scroll_to(ClutterActor *inActor,
 		}
 }
 
+/* Ensure that a child of a view is visible by scrolling if needed */
+static void _xfdashboard_viewpad_on_view_ensure_visible(XfdashboardViewpad *self,
+														ClutterActor *inActor,
+														gpointer inUserData)
+{
+	XfdashboardViewpadPrivate	*priv;
+	XfdashboardView				*view;
+	ClutterVertex				origin;
+	ClutterVertex				transformedUpperLeft;
+	ClutterVertex				transformedLowerRight;
+	gfloat						x, y, w, h;
+	gboolean					needScrolling;
+
+	g_return_if_fail(XFDASHBOARD_IS_VIEWPAD(self));
+	g_return_if_fail(CLUTTER_IS_ACTOR(inActor));
+	g_return_if_fail(XFDASHBOARD_IS_VIEW(inUserData));
+
+	priv=self->priv;
+	view=XFDASHBOARD_VIEW(inUserData);
+	needScrolling=FALSE;
+	origin.z=0.0f;
+
+	/* Get position and size of view but respect scrolled position */
+	if(view==priv->activeView)
+	{
+		x=xfdashboard_scrollbar_get_value(XFDASHBOARD_SCROLLBAR(priv->hScrollbar));
+		y=xfdashboard_scrollbar_get_value(XFDASHBOARD_SCROLLBAR(priv->vScrollbar));
+		clutter_actor_get_size(CLUTTER_ACTOR(self), &w, &h);
+	}
+		else
+		{
+			if(clutter_actor_has_clip(CLUTTER_ACTOR(view)))
+			{
+				clutter_actor_get_clip(CLUTTER_ACTOR(view), &x, &y, &w, &h);
+			}
+				else
+				{
+					x=y=0.0f;
+					clutter_actor_get_size(CLUTTER_ACTOR(view), &w, &h);
+				}
+		}
+
+	/* Check that upper left point of actor is visible otherwise set flag for scrolling */
+	origin.x=origin.y=0.0f;
+	clutter_actor_apply_relative_transform_to_point(inActor, CLUTTER_ACTOR(view), &origin, &transformedUpperLeft);
+	if(transformedUpperLeft.x<x ||
+		transformedUpperLeft.x>(x+w) ||
+		transformedUpperLeft.y<y ||
+		transformedUpperLeft.y>(y+h))
+	{
+		needScrolling=TRUE;
+	}
+
+	/* Check that lower right point of actor is visible otherwise set flag for scrolling */
+	clutter_actor_get_size(inActor, &origin.x, &origin.y);
+	clutter_actor_apply_relative_transform_to_point(inActor, CLUTTER_ACTOR(view), &origin, &transformedLowerRight);
+	if(transformedLowerRight.x<x ||
+		transformedLowerRight.x>(x+w) ||
+		transformedLowerRight.y<y ||
+		transformedLowerRight.y>(y+h))
+	{
+		needScrolling=TRUE;
+	}
+
+	/* Check if we need to scroll */
+	if(needScrolling)
+	{
+		gfloat					distanceUpperLeft;
+		gfloat					distanceLowerRight;
+
+		/* Find shortest way to scroll and then scroll */
+		distanceUpperLeft=sqrtf(powf(transformedUpperLeft.x-x, 2.0f)+powf(transformedUpperLeft.y-y, 2.0f));
+		distanceLowerRight=sqrtf(powf(transformedLowerRight.x-(x+w), 2.0f)+powf(transformedLowerRight.y-(y+h), 2.0f));
+
+		if(distanceUpperLeft<=distanceLowerRight)
+		{
+			_xfdashboard_viewpad_on_view_scroll_to(self, transformedUpperLeft.x, transformedUpperLeft.y, view);
+		}
+			else
+			{
+				_xfdashboard_viewpad_on_view_scroll_to(self, transformedUpperLeft.x, transformedLowerRight.y-h, view);
+			}
+	}
+}
+
 /* Create view of given type and add to this actor */
 static void _xfdashboard_viewpad_add_view(XfdashboardViewpad *self, GType inViewType)
 {
@@ -482,6 +569,7 @@ static void _xfdashboard_viewpad_add_view(XfdashboardViewpad *self, GType inView
 	clutter_actor_add_child(CLUTTER_ACTOR(self), CLUTTER_ACTOR(view));
 	g_signal_connect_swapped(view, "allocation-changed", G_CALLBACK(_xfdashboard_viewpad_on_allocation_changed), self);
 	g_signal_connect_swapped(view, "scroll-to", G_CALLBACK(_xfdashboard_viewpad_on_view_scroll_to), self);
+	g_signal_connect_swapped(view, "ensure-visible", G_CALLBACK(_xfdashboard_viewpad_on_view_ensure_visible), self);
 	g_signal_connect_swapped(view, "disabled", G_CALLBACK(_xfdashboard_viewpad_on_view_disabled), self);
 	g_signal_connect_swapped(view, "enabled", G_CALLBACK(_xfdashboard_viewpad_on_view_enabled), self);
 	g_signal_emit(self, XfdashboardViewpadSignals[SIGNAL_VIEW_ADDED], 0, view);
@@ -766,6 +854,156 @@ static void _xfdashboard_viewpad_allocate(ClutterActor *self,
 	}
 }
 
+/* IMPLEMENTATION: Interface XfdashboardFocusable */
+
+/* Determine if actor can get the focus */
+static gboolean _xfdashboard_viewpad_focusable_can_focus(XfdashboardFocusable *inFocusable)
+{
+	XfdashboardViewpad			*self;
+	XfdashboardViewpadPrivate	*priv;
+	gboolean					canFocus;
+
+	g_return_val_if_fail(XFDASHBOARD_IS_FOCUSABLE(inFocusable), FALSE);
+	g_return_val_if_fail(XFDASHBOARD_IS_VIEWPAD(inFocusable), FALSE);
+
+	self=XFDASHBOARD_VIEWPAD(inFocusable);
+	priv=self->priv;
+
+	/* Set current focusable result to FALSE (not focusable). It will be set
+	 * to TRUE automatically if current active view is focusable and its
+	 * virtual function will also return TRUE.
+	 */
+	canFocus=FALSE;
+
+	/* Viewpad is just a proxy for the current active view.
+	 * So check if current active view is focusable and call its
+	 * virtual function.
+	 */
+	if(priv->activeView &&
+		XFDASHBOARD_IS_FOCUSABLE(priv->activeView))
+	{
+		canFocus=xfdashboard_focusable_can_focus(XFDASHBOARD_FOCUSABLE(priv->activeView));
+	}
+
+	/* Return focusable state */
+	return(canFocus);
+}
+
+/* Set focus to actor */
+static void _xfdashboard_viewpad_focusable_set_focus(XfdashboardFocusable *inFocusable)
+{
+	XfdashboardViewpad				*self;
+	XfdashboardViewpadPrivate		*priv;
+	XfdashboardFocusableInterface	*selfIface;
+	XfdashboardFocusableInterface	*parentIface;
+
+	g_return_if_fail(XFDASHBOARD_IS_FOCUSABLE(inFocusable));
+	g_return_if_fail(XFDASHBOARD_IS_VIEWPAD(inFocusable));
+
+	self=XFDASHBOARD_VIEWPAD(inFocusable);
+	priv=self->priv;
+
+	/* Viewpad is just a proxy for the current active view.
+	 * So check if current active view is focusable and call its
+	 * virtual function.
+	 */
+	if(priv->activeView &&
+		XFDASHBOARD_IS_FOCUSABLE(priv->activeView))
+	{
+		/* Call virtual function of view to set focus */
+		xfdashboard_focusable_set_focus(XFDASHBOARD_FOCUSABLE(priv->activeView));
+
+		/* Call parent class interface function of this actor */
+		selfIface=XFDASHBOARD_FOCUSABLE_GET_IFACE(inFocusable);
+		parentIface=g_type_interface_peek_parent(selfIface);
+
+		if(parentIface && parentIface->set_focus)
+		{
+			parentIface->set_focus(inFocusable);
+		}
+	}
+}
+
+/* Unset focus from actor */
+static void _xfdashboard_viewpad_focusable_unset_focus(XfdashboardFocusable *inFocusable)
+{
+	XfdashboardViewpad				*self;
+	XfdashboardViewpadPrivate		*priv;
+	XfdashboardFocusableInterface	*selfIface;
+	XfdashboardFocusableInterface	*parentIface;
+
+	g_return_if_fail(XFDASHBOARD_IS_FOCUSABLE(inFocusable));
+	g_return_if_fail(XFDASHBOARD_IS_VIEWPAD(inFocusable));
+
+	self=XFDASHBOARD_VIEWPAD(inFocusable);
+	priv=self->priv;
+
+	/* Viewpad is just a proxy for the current active view.
+	 * So check if current active view is focusable and call its
+	 * virtual function.
+	 */
+	if(priv->activeView &&
+		XFDASHBOARD_IS_FOCUSABLE(priv->activeView))
+	{
+		/* Call virtual function of view to unset focus */
+		xfdashboard_focusable_unset_focus(XFDASHBOARD_FOCUSABLE(priv->activeView));
+
+		/* Call parent class interface function of this actor */
+		selfIface=XFDASHBOARD_FOCUSABLE_GET_IFACE(inFocusable);
+		parentIface=g_type_interface_peek_parent(selfIface);
+
+		if(parentIface && parentIface->unset_focus)
+		{
+			parentIface->unset_focus(inFocusable);
+		}
+	}
+}
+
+/* Virtual function "handle_key_event" was called */
+static gboolean _xfdashboard_viewpad_focusable_handle_key_event(XfdashboardFocusable *inFocusable,
+																const ClutterEvent *inEvent)
+{
+	XfdashboardViewpad			*self;
+	XfdashboardViewpadPrivate	*priv;
+	gboolean					handledEvent;
+
+	g_return_val_if_fail(XFDASHBOARD_IS_FOCUSABLE(inFocusable), CLUTTER_EVENT_PROPAGATE);
+	g_return_val_if_fail(XFDASHBOARD_IS_VIEWPAD(inFocusable), CLUTTER_EVENT_PROPAGATE);
+
+	self=XFDASHBOARD_VIEWPAD(inFocusable);
+	priv=self->priv;
+
+	/* Set handled key eventto CLUTTER_EVENT_PROPAGATE. It might be set to
+	 * CLUTTER_EVENT_STOP if current active view is focusable and it handled
+	 * the key event by its virtual function.
+	 */
+	handledEvent=CLUTTER_EVENT_PROPAGATE;
+
+	/* Viewpad is just a proxy for the current active view.
+	 * So check if current active view is focusable and call its
+	 * virtual function.
+	 */
+	if(priv->activeView &&
+		XFDASHBOARD_IS_FOCUSABLE(priv->activeView))
+	{
+		handledEvent=xfdashboard_focusable_handle_key_event(XFDASHBOARD_FOCUSABLE(priv->activeView), inEvent);
+	}
+
+	/* Return focusable state */
+	return(handledEvent);
+}
+
+/* Interface initialization
+ * Set up default functions
+ */
+void _xfdashboard_viewpad_focusable_iface_init(XfdashboardFocusableInterface *iface)
+{
+	iface->can_focus=_xfdashboard_viewpad_focusable_can_focus;
+	iface->set_focus=_xfdashboard_viewpad_focusable_set_focus;
+	iface->unset_focus=_xfdashboard_viewpad_focusable_unset_focus;
+	iface->handle_key_event=_xfdashboard_viewpad_focusable_handle_key_event;
+}
+
 /* IMPLEMENTATION: GObject */
 
 /* Dispose this object */
@@ -886,21 +1124,21 @@ static void xfdashboard_viewpad_class_init(XfdashboardViewpadClass *klass)
 							_("The spacing between views and scrollbars"),
 							0.0f, G_MAXFLOAT,
 							0.0f,
-							G_PARAM_READWRITE);
+							G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
 	XfdashboardViewpadProperties[PROP_ACTIVE_VIEW]=
 		g_param_spec_object("active-view",
 								_("Active view"),
 								_("The current active view in viewpad"),
 								XFDASHBOARD_TYPE_VIEW,
-								G_PARAM_READABLE);
+								G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
 
 	XfdashboardViewpadProperties[PROP_HSCROLLBAR_VISIBLE]=
 		g_param_spec_boolean("horizontal-scrollbar-visible",
 								_("Horizontal scrollbar visibility"),
 								_("This flag indicates if horizontal scrollbar is visible"),
 								FALSE,
-								G_PARAM_READABLE);
+								G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
 
 	XfdashboardViewpadProperties[PROP_HSCROLLBAR_POLICY]=
 		g_param_spec_enum("horizontal-scrollbar-policy",
@@ -908,14 +1146,14 @@ static void xfdashboard_viewpad_class_init(XfdashboardViewpadClass *klass)
 							_("The policy for horizontal scrollbar controlling when it is displayed"),
 							XFDASHBOARD_TYPE_POLICY,
 							XFDASHBOARD_POLICY_AUTOMATIC,
-							G_PARAM_READWRITE);
+							G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
 	XfdashboardViewpadProperties[PROP_VSCROLLBAR_VISIBLE]=
 		g_param_spec_boolean("vertical-scrollbar-visible",
 								_("Vertical scrollbar visibility"),
 								_("This flag indicates if vertical scrollbar is visible"),
 								FALSE,
-								G_PARAM_READABLE);
+								G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
 
 	XfdashboardViewpadProperties[PROP_VSCROLLBAR_POLICY]=
 		g_param_spec_enum("vertical-scrollbar-policy",
@@ -923,7 +1161,7 @@ static void xfdashboard_viewpad_class_init(XfdashboardViewpadClass *klass)
 							_("The policy for vertical scrollbar controlling when it is displayed"),
 							XFDASHBOARD_TYPE_POLICY,
 							XFDASHBOARD_POLICY_AUTOMATIC,
-							G_PARAM_READWRITE);
+							G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
 	g_object_class_install_properties(gobjectClass, PROP_LAST, XfdashboardViewpadProperties);
 
@@ -1017,7 +1255,7 @@ static void xfdashboard_viewpad_init(XfdashboardViewpad *self)
 	priv=self->priv=XFDASHBOARD_VIEWPAD_GET_PRIVATE(self);
 
 	/* Set up default values */
-	priv->viewManager=XFDASHBOARD_VIEW_MANAGER(g_object_ref(xfdashboard_view_manager_get_default()));
+	priv->viewManager=xfdashboard_view_manager_get_default();
 	priv->activeView=NULL;
 	priv->spacing=0.0f;
 	priv->hScrollbarVisible=FALSE;
